@@ -100,6 +100,7 @@ def find_new_substring(s1, s2):
         return s2.replace(s1, '', 1)
     return s2
 
+
 class Agent:
 
     def __init__(self, model=None, sys_core_template=None, agent=None, clear_model=False, commands=[], context=None):
@@ -142,9 +143,15 @@ class Agent:
         #await asyncio.sleep(1)
 
     async def handle_cmds(self, cmd_name, cmd_args, json_cmd=None, context=None):
-        if 'finished_conversation' in context.data and context.data['finished_conversation']:
-            logger.warning("Conversation is finished, not executing command")
-            print("\033[91mConversation is finished, not executing command\033[0m")
+        # Check both permanent finish and temporary cancellation
+        if context.data.get('cancel_current_turn'):
+            logger.warning("Turn cancelled, not executing command")
+            print("\033[91mTurn cancelled, not executing command\033[0m")
+            raise asyncio.CancelledError("Turn cancelled")
+        
+        if context.data.get('finished_conversation'):
+            logger.warning("Conversation finished, not executing command")
+            print("\033[91mConversation finished, not executing command\033[0m")
             return None
 
         logger.info("Command execution: {command}", command=cmd_name)
@@ -295,17 +302,25 @@ class Agent:
 
             logger.debug(f"commands: {commands}, partial_cmd: {partial_cmd}")
 
-            if 'finished_conversation' in context.data and context.data['finished_conversation']:
+            # Check for cancellation (either permanent or current turn)
+            if context.data.get('finished_conversation') or context.data.get('cancel_current_turn'):
                 logger.warning("Conversation is finished or halted, exiting stream parsing")
                 debug_box(f"""Conversation is finished or halted, exiting stream""")
                 debug_box(str(context))
-                # stream is actually a generator
+                
+                # Add partial command to chat log if present
                 if partial_cmd is not None:
                     cmd_name = next(iter(partial_cmd))
                     if cmd_name in ["say", "json_encoded_md", "think"]:
                         context.chat_log.add_message({"role": "assistant", "content": str(partial_cmd[cmd_name])})
                     else:
                         context.chat_log.add_message({"role": "assistant", "content": str(partial_cmd) + "(Interrupted)"})
+                
+                # Clear the temporary cancel flag so next turn can proceed
+                if 'cancel_current_turn' in context.data:
+                    del context.data['cancel_current_turn']
+                    await context.save_context()
+                
                 try:
                     stream.close()
                 except Exception as e:
@@ -334,11 +349,32 @@ class Agent:
                         cmd_args = cmd[cmd_name]
                         logger.debug(f"Processing command: {cmd}")
                         await context.partial_command(cmd_name, json.dumps(cmd_args), cmd_args)
+ 
+                        cmd_task = asyncio.create_task(
+                            self.handle_cmds(cmd_name, cmd_args, json_cmd=json.dumps(cmd), context=context)
+                        )
+                        context.data['active_command_task'] = cmd_task
+                        try:
+                            result = await cmd_task
+                        except asyncio.CancelledError:
+                            raise  # Propagate cancellation up
+                        finally:
+                            # Clear the task from context once it's done or cancelled
+                            if context.data.get('active_command_task') == cmd_task:
+                                del context.data['active_command_task']
 
-                        result = await self.handle_cmds(cmd_name, cmd_args, json_cmd=json.dumps(cmd), context=context)
                         await context.command_result(cmd_name, result)
                         sys_header = "Note: tool command results follow, not user replies" 
                         sys_header = ""
+
+                        if result == "SYSTEM: WARNING - Command interrupted!\n\n":
+                            logger.warning("Command was interrupted. Stopping processing.")
+                            await context.chat_log.drop_last('assistant')
+                            await asyncio.sleep(0.5)
+                            break
+                            return results, full_cmds
+
+
                         full_cmds.append({ "SYSTEM": sys_header, "cmd": cmd_name, "args": cmd_args, "result": result})
                         if result is not None:
                             results.append({"SYSTEM": sys_header, "cmd": cmd_name, "args": { "omitted": "(see command msg.)"}, "result": result})
@@ -414,15 +450,7 @@ class Agent:
             "formatted_datetime": formatted_time,
             "context_data": self.context.data
         }
-        # is say in the command_manager
-        if 'say' in command_manager.functions.keys():
-            print("I found say! in the functions!")
-        else:
-            print("Say is not in the functions!")
-        if 'say' in data['command_docs'].keys():
-            print("I found say in the command docs!")
-    
-        # we need to be doubly sure to remove anything from command_docs that is not in command_manager.functions.keys()
+   
         for cmd in data['command_docs']:
             if cmd not in command_manager.functions.keys():
                 print("Removing " + cmd + " from command_docs")
@@ -457,6 +485,7 @@ class Agent:
             tmp_data = { "messages": new_messages }
             debug_box("Filtering messages")
             #debug_box(tmp_data)
+
             tmp_data = await pipeline_manager.filter_messages(tmp_data, context=context)
             new_messages = tmp_data['messages']
         except Exception as e:
@@ -471,18 +500,30 @@ class Agent:
         if not isinstance(context.agent, dict):
             context.agent = await get_agent_data(context.agent, context=context)
 
+        if 'max_tokens' in context.agent and context.agent['max_tokens'] is not None and context.agent['max_tokens'] != '':
+            logger.info(f"Using agent max tokens {max_tokens}")
+            max_tokens = context.agent['max_tokens']
+        else:
+            logger.info(f"Using default max tokens {max_tokens}")
+
         if model is None:
             if 'service_models' in context.agent and context.agent['service_models'] is not None:
                 if context.agent['service_models'].get('stream_chat', None) is None:
                     model = os.environ.get("DEFAULT_LLM_MODEL")
-
+        
+        # we need to be able to abort this task if necessary
         stream = await context.stream_chat(model,
                                         temperature=temperature,
                                         max_tokens=max_tokens,
                                         messages=new_messages,
                                         context=context)
-
-        ret, full_cmds = await self.parse_cmd_stream(stream, context)
+        
+        try:
+            ret, full_cmds = await self.parse_cmd_stream(stream, context)
+        except asyncio.CancelledError:
+            logger.info("Command stream parsing cancelled")
+            raise  # Propagate cancellation
+        
         logger.debug("System message was:")
         logger.debug(await self.render_system_msg())
 
