@@ -8,6 +8,7 @@ from typing import List
 from lib.utils.dataurl import dataurl_to_pil
 from .models import MessageParts
 from coreplugins.agent import agent
+from coreplugins.agent.speech_to_speech import SpeechToSpeechAgent
 from lib.utils.debug import debug_box
 import os
 import sys
@@ -22,6 +23,10 @@ from io import BytesIO
 import base64
 import nanoid
 sse_clients = {}
+from lib.chatcontext import get_context
+
+# Track active processing tasks per session
+active_tasks = {}
 
 @service()
 async def prompt(model: str, instructions: str, temperature=0, max_tokens=400, json=False, context=None):
@@ -67,7 +72,7 @@ def results_output(results):
     text = ""
     for result in results:
         if 'output' in result['args']:
-            return result['args']['output']
+            return str(result['args']['output'])
 
 def results_text_output(results):
     text = ""
@@ -134,6 +139,9 @@ async def run_task(instructions: str, agent_name:str = None, user:str = None, lo
 
     while retried < retries:
         [results, full_results] = await send_message_to_agent(context.log_id, instructions, context=context)
+        print('#####################################################33')
+        print("Full results: ", full_results)
+        print("Results: ", results)
         text = results_output(full_results)
         if text == "":
             retried += 1
@@ -164,6 +172,11 @@ async def init_chat_session(user:str, agent_name: str, log_id: str, context=None
         print("context.agent_name: ", context.agent_name)
         await context.save_context()
     print("initiated_chat_session: ", log_id, agent_name, context.agent_name, context.agent)
+
+    if 'live' in context.agent['stream_chat'] or 'realtime' in context.agent['stream_chat']:
+        agent = SpeechToSpeechAgent(agent_name, context=context)
+        await agent.connect()
+
     return log_id
 
 @service()
@@ -216,13 +229,62 @@ def process_result(result, formatted_results):
 
     return formatted_results
 
+# Deprecated - use active_tasks instead
+in_progress = {}
+
+
 @service()
-async def send_message_to_agent(session_id: str, message: str | List[MessageParts], max_iterations=35, context=None, user=None):
-    if os.environ.get("MR_MAX_ITERATIONS") is not None:
-        max_iterations = int(os.environ.get("MR_MAX_ITERATIONS"))
+async def cancel_and_wait(session_id: str, user:str, context=None):
+    global in_progress, active_tasks
+    existing_task = active_tasks.get(session_id)
+
+    if not in_progress.get(session_id, False):
+        print(f"SEND_MESSAGE No active processing for session {session_id} to cancel.")
+        return
+    try:
+        print(f"SEND_MESSAGE Cancelling active task for session {session_id}...")
+        existing_context = await get_context(session_id, user)
+        existing_context.data['cancel_current_turn'] = True
+        existing_context.data['finished_conversation'] = True
+        existing_context.save_context()
+        # Cancel any active command task
+        if 'active_command_task' in existing_context.data:
+            cmd_task = existing_context.data['active_command_task']
+            if cmd_task and not cmd_task.done():
+                cmd_task.cancel()
+        
+        await existing_context.save_context()
+    except Exception as e:
+        print(f"SEND_MESSAGE Error setting cancellation flags: {e}")
+    
+    # Cancel the main task
+    existing_task.cancel()
+    
+    # Wait for it to actually finish (with timeout)
+    try:
+        await asyncio.wait_for(existing_task, timeout=2.0)
+    except (asyncio.CancelledError, asyncio.TimeoutError):
+        pass  # Expected
+   
+    start_wait = time.time()
+    while in_progress.get(session_id, False) and (time.time() - start_wait) < 5.0:
+        print(f"SEND_MESSAGE Waiting for cancellation of session {session_id} to complete...")
+        await asyncio.sleep(0.025)
+
+    print(f"SEND_MESSAGE Cancellation complete for session {session_id}")
+
+@service()
+async def send_message_to_agent(session_id: str, message: str | List[MessageParts], max_iterations=35, context=None, user=None,
+                                assume_wait_for_task_result=False):
+    global in_progress, active_tasks
+    
+    # Check if there's an active task for this session
+    existing_task = active_tasks.get(session_id)
+
     if not user:
         # check context
         if not context.username:
+            print("SEND_MESSAGE No user provided and context has no username")
             raise Exception("User required")
         else:
             user = {"user": context.username }
@@ -230,20 +292,46 @@ async def send_message_to_agent(session_id: str, message: str | List[MessagePart
         if hasattr(user, "dict"):
             user = user.dict()
 
+    context.data['cancel_current_turn'] = False
+    context.data['finished_conversation'] = False
+    await context.save_context()
+
+    in_progress[session_id] = True
+    await asyncio.sleep(0.05)
+
+    print('b')
+    if os.environ.get("MR_MAX_ITERATIONS") is not None:
+        max_iterations = int(os.environ.get("MR_MAX_ITERATIONS"))
     try:
         if type(message) is list:
             message = [m.dict() for m in message]
 
         if session_id is None or session_id == "" or message is None or message == "":
-            print("Invalid session_id or message")
+            print("SEND_MESSAGE Invalid session_id or message")
             return []
 
+        # Create the main processing task and store it
+        processing_task = asyncio.current_task()
+        active_tasks[session_id] = processing_task
+
+        print("SEND_MESSAGE proceeding with message processing")
         print("send_message_to_agent: ", session_id, message, max_iterations)
         if context is None:
             context = ChatContext(command_manager, service_manager, user)
             await context.load_context(session_id)
 
-        agent_ = agent.Agent(agent=context.agent)
+        if 'live' in context.agent['stream_chat'] or 'realtime' in context.agent['stream_chat'] or assume_wait_for_task_result==True:
+            agent_ = SpeechToSpeechAgent(context.agent_name, context=context)
+            print('Using SpeechToSpeechAgent for live/realtime chat')
+            print()
+            print()
+            print()
+            print('message: ', message)
+            [results, full_results] = await agent_.send_message(message, wait_for_task_result=True, context=context)
+            return [results, full_results]
+        else:
+            agent_ = agent.Agent(agent=context.agent)
+
         if user is not None and hasattr(user, "keys"):
             for key in user.keys():
                 context.data[key] = user[key]
@@ -372,9 +460,12 @@ async def send_message_to_agent(session_id: str, message: str | List[MessagePart
                     context.chat_log.add_message({"role": "user", "content": formatted_results})
                     results.append(out_results) 
                 else:
-                    print("Processing iteration: ", iterations, "no message added")
+                    print("SEND_MESSAGE Processing iteration: ", iterations, "no message added")
                 if context.data.get('finished_conversation') is True:
                     termcolor.cprint("Finished conversation, exiting send_message_to_agent", "red")
+                    if context.data.get('task_result') is not None:
+                        task_result = context.data.get('task_result')
+                        full_results.append({ "cmd": "task_result", "args": { "result": task_result } })
                     continue_processing = False
             except Exception as e:
                 continue_processing = False
@@ -398,10 +489,23 @@ async def send_message_to_agent(session_id: str, message: str | List[MessagePart
         print("Exiting send_message_to_agent: ", session_id, message, max_iterations)
 
         await context.finished_chat()
+        in_progress.pop(session_id, None)
+        active_tasks.pop(session_id, None)
+        if len(results) == 0:
+            if context.data.get('task_result') is not None:
+                task_result = context.data.get('task_result')
+                results.append(task_result)
+        print("SEND_MESSAGE Done")
         return [results, full_results]
+    except asyncio.CancelledError:
+        print(f"Task cancelled for session {session_id}")
+        in_progress.pop(session_id, None)
+        active_tasks.pop(session_id, None)
+        raise  # Re-raise to properly handle cancellation
     except Exception as e:
         print("Error in send_message_to_agent: ", e)
         print(traceback.format_exc())
+        in_progress.pop(session_id, None)
         return []
 
 @pipe(name='process_results', priority=5)
@@ -485,3 +589,83 @@ async def command_result(command: str, result, context=None):
     agent_ = context.agent
     await context.agent_output("command_result", { "command": command, "result": result, "persona": agent_['persona']['name'] })
 
+@service()
+async def backend_user_message(message: str, context=None):
+    """
+    Signal the frontend to display a user message.
+    """
+    agent_ = context.agent
+    persona = 'user'
+    await context.agent_output("backend_user_message", { 
+        "content": message, 
+        "sender": "user",
+        "persona": persona
+    })
+
+@service()
+async def backend_assistant_message(message: str, context=None):
+    """
+    Signal the frontend to display an assistant message.
+    """
+    agent_ = context.agent
+    persona = 'assistant'
+    await context.agent_output("backend_assistant_message", { 
+        "content": message, 
+        "sender": "assistant",
+        "persona": persona
+    })
+
+@service()
+async def cancel_active_response(log_id: str, context=None):
+    """
+    Cancel active AI response for eager end of turn processing.
+    Sets the finished_conversation flag to stop the agent processing loop.
+    """
+    if context is None:
+        # Get context from log_id - we need the username, but for SIP calls it might be 'system'
+        # Try to load context, fallback to system user if needed
+        try:
+            context = await get_context(log_id, 'system')
+        except Exception as e:
+            print(f"Error getting context for cancellation: {e}")
+            return {"status": "error", "message": f"Could not load context: {e}"}
+    
+    # Set flag to stop current processing loop iteration
+    # But don't permanently mark conversation as finished - just this turn
+    context.data['cancel_current_turn'] = True
+    
+    # DEBUG TRACE
+    print("\033[91;107m[DEBUG TRACE 5/6] Core cancel_active_response service executed.\033[0m")
+    
+    # Cancel any active TTS streams (ElevenLabs)
+    try:
+        # Import here to avoid circular dependency
+        from mr_eleven_stream.mod import _active_tts_streams
+        for stream_id, stop_event in list(_active_tts_streams.items()):
+            stop_event.set()
+            logger.info(f"Cancelled TTS stream {stream_id}")
+            print("\033[91;107m[DEBUG TRACE 5.5/6] Cancelled active TTS stream.\033[0m")
+    except ImportError:
+        logger.debug("ElevenLabs TTS plugin not available for cancellation")
+    
+    # Also, cancel any active command task (like speak())
+    if 'active_command_task' in context.data:
+        active_task = context.data['active_command_task']
+        if active_task and not active_task.done():
+            try:
+                active_task.cancel()
+                # DEBUG TRACE
+                print("\033[91;107m[DEBUG TRACE 6/6] Active command task found and cancelled.\033[0m")
+                print(f"Cancelled active command task for session {log_id}")
+                print(f"Removing last assistant message from chat log for session {log_id}")
+                await context.chat_log.drop_last('assistant')
+
+            except Exception as e:
+                print(f"Error cancelling active command task: {e}")
+
+ 
+    await context.save_context()
+    
+    print(f"Cancelled active response for session {log_id}")
+    return {"status": "cancelled", "log_id": log_id}
+
