@@ -593,3 +593,55 @@ You are working on {task_context}."""
     
     return await command_manager.delegate_task(instructions, agent_name, context=context)
 
+
+
+@command()
+async def create_checklist(tasks, title=None, replace=True, context=None):
+    """
+    Create a new checklist dynamically from a list of task descriptions.
+    
+    Parameters:
+    - tasks: Required. List of task description strings
+    - title: Optional. Title for the checklist (for display purposes)
+    - replace: Optional. Whether to replace existing checklist (default: True)
+    
+    Example:
+    { "create_checklist": { 
+        "tasks": [
+            "Research the topic",
+            "Create outline", 
+            "Write first draft",
+            "Review and edit"
+        ],
+        "title": "Article Writing Process"
+    }}
+    """
+    if context is None:
+        return "_Context is required._"
+    
+    if not tasks or not isinstance(tasks, list):
+        return "_Tasks parameter must be a non-empty list of task descriptions._"
+    
+    # Check if we should replace existing checklist
+    st = _state(context)
+    if not replace and st["tasks"]:
+        return "_Checklist already exists. Use replace=True to overwrite it._"
+    
+    # Convert task list to markdown checklist format
+    markdown_lines = []
+    if title:
+        markdown_lines.append(f"# {title}\n")
+    
+    for task_desc in tasks:
+        if not isinstance(task_desc, str):
+            return "_All tasks must be strings._"
+        markdown_lines.append(f"- [ ] {task_desc.strip()}")
+    
+    markdown_text = "\n".join(markdown_lines)
+    
+    # Use existing load_checklist function to parse and store
+    load_checklist(markdown_text, context)
+    
+    # Build response
+    title_text = f" '{title}'" if title else ""
+    return f"Created checklist{title_text} with {len(tasks)} tasks.\n\n{_format_checklist_status(context)}"
